# Parcel automatic routing plug-in

## Intro

Scan the directory's Vue file, and automatically generate routing according to the file and directory structure. The rules are simple.

* the initial document (Vue file, the same below) represents the address bar parameter (params).

* the file at the beginning of the currency symbol ($) represents lazy load routing.

* the file at the beginning of the double underline represents the missing file.

* extendRouter.js adds additional information to the directory generated by the directory (information other than path and component).



## Use

src/router/index.js

```javascript
Import Vue from'vue'
Import Router from'vue-router'
Import Autorouter from'./view.autorouter'

Vue.use (Router)

Const router = new Router ({
  Routes: [autorouter]
})
```

src/router/view.autorouter

```json
{
  "DirPath": "../views"
}
```

The Autorouter file supports two parameters

- dirPath: to scan the folder, must fill

- the name of the folder that ignore: does not scan, the default value is ['assets','components']



## Special document conventions

- main.vue: main entrance, optional

As a container for all the following routes, the file must be written in vouter-view, and the default main entry (a component that contains only vouter-view tags) is automatically inserted without time.

- index.vue: home page, optional

As a default subrouter, the path is not accessible when the file is not available (the lower route of the path can be accessed).

- _.vue: processing *, optional

Used to handle all routes in this path that are not in the routing rules (typically used as 404 pages), note that the file still passes through the main. Vue

- extendRouter.js extra routing information, optional

Provide other routing information besides path and component

Note: the names of main.vue and index.vue must be all lowercase.

## URL params

All the directories or files that begin with the underscore are automatically processed into URL parameters. The parameter is called the character after the underline.

- only one underscore(_) can be allow, and two or more delegates mean ignore this file.

- in order to avoid routing to unexpected branches, the following conventions are made for subfiles and subdirectories under directory.

- there is only one directory that starts with a word.

- Only one Vue file (excluding. Vue,. Vue processed into *) can start with and all other Vue files will be ignored when such a file exists

Index.vue or $index.vue in the directory will be ignored when files and directories are in the beginning.

- other possible routing conflicts, such as the directory with the beginning of the directory and the normal brotherhood, they all have step1.vue below, which will cause one of the routes to go and need to solve them themselves.

## Ignore directories and files

- when scanning, you will automatically ignore the directory or Vue file beginning with the "double" underline.

## Async and subcontract

- subcontract by file: the Vue file starting with $represents the asynchronous component, which automatically subcontracts.

- Packet by directory: Create a $main. Vue file, and the underlying directory and all subdirectories will be packaged in one package. The main. Vue file must contain a router-view

## Add routing additional information

ExtendRouter.js processing (filter.js processing path and component will be ignored)

The file returns a method on the path where the routing is, which is called automatically by the automatic routing plug-in. The method passes into the routing path and returns to the desired path configuration.

## Note

- no Vue files and directories with the same name can be processed. If so, the files are processed and the directory is ignored.

- $_.vue and _$.vue are meaningless. Directory is meaningless and will be ignored.

- when the asynchronous component is used, only one $ can appear.

- When using asynchronous and URL parameters at the same time, please start with _$